﻿using System.Windows;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 셀 패딩 그리드
    /// </summary>
    public class CellPaddingGrid : Grid
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 셀 패딩 속성
        /// </summary>
        public static readonly DependencyProperty CellPaddingProperty = DependencyProperty.Register
        (
            "CellPadding",
            typeof(Thickness),
            typeof(CellPaddingGrid),
            new FrameworkPropertyMetadata
            (
                new Thickness(0.0),
                FrameworkPropertyMetadataOptions.AffectsArrange,
                CellPaddingPropertyChangedCallback
            )
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 셀 패딩 - CellPadding

        /// <summary>
        /// 셀 패딩
        /// </summary>
        public Thickness CellPadding
        {
            get 
            {
                return (Thickness)GetValue(CellPaddingProperty);
            }
            set
            {
                SetValue(CellPaddingProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 마진 적용하기 - ApplyMargin(grid, element)

        /// <summary>
        /// 마진 적용하기
        /// </summary>
        /// <param name="grid">그리드</param>
        /// <param name="element">UI 엘리먼트</param>
        public static void ApplyMargin(CellPaddingGrid grid, UIElement element)
        {
            FrameworkElement frameworkElement = element as FrameworkElement;

            Thickness cellPadding = grid.CellPadding;

            CellPaddingGrid grid2 = element as CellPaddingGrid;

            if(grid2 != null)
            {
                grid2.CellPadding = cellPadding;
            }
            else
            {
                frameworkElement.Margin = cellPadding;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 셀 패딩 속성 변경시 콜백 처리하기 - CellPaddingPropertyChangedCallback(d, e)

        /// <summary>
        /// 셀 패딩 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void CellPaddingPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            CellPaddingGrid grid = d as CellPaddingGrid;

            foreach(UIElement element in grid.Children)
            {
                ApplyMargin(grid, element);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 비주얼 자식 변경시 처리하기 - OnVisualChildrenChanged(addVisual, removeVisual)

        /// <summary>
        /// 비주얼 자식 변경시 처리하기
        /// </summary>
        /// <param name="addVisual">추가 비주얼</param>
        /// <param name="removeVisual">제거 비주얼</param>
        protected override void OnVisualChildrenChanged(DependencyObject addVisual, DependencyObject removeVisual)
        {
            FrameworkElement childElement = addVisual as FrameworkElement;

            ApplyMargin(this, childElement);

            base.OnVisualChildrenChanged(addVisual, removeVisual);
        }

        #endregion
    }  
}